//================================================================= includes ===
#include <math.h>
#include "movelin.h"

//================================================================== globals ===
uint16_t accelerationStop; //volatile uint16_t accelerationStop;
uint16_t decelerationStart;
uint16_t goal;
uint16_t currentStep; //volatile uint16_t currentStep;
volatile uint16_t counterValue;
uint8_t refreshCounterValue; //volatile uint8_t refreshCounterValue;
uint8_t busy; //volatile uint8_t busy;

//====================================================================== isr ===
ISR(TIMER1_COMPA_vect) {

	if (refreshCounterValue) {
		if (currentStep <= accelerationStop) { //acceleration mode
			counterValue = counterValue	- ((2 * counterValue) / (4 * (currentStep + 1) + 1));
			counterValue = counterValue / 2;
		} else if ((currentStep) >= decelerationStart) { //deceleration mode
			accelerationStop++;
			counterValue = ((4 * accelerationStop * counterValue + counterValue) / (4 * accelerationStop - 1));
			counterValue = counterValue / 2;
			accelerationStop -= 2;
		}

		OCR1AH = (uint8_t) (counterValue >> 8);
		OCR1AL = (uint8_t) (counterValue & 0xFF);

		currentStep++;
		if (currentStep >= goal) {
			TIMSK &= ~(1 << OCIE1A);
			busy = 0;
		}
		refreshCounterValue = 0;
		PORTB &= ~(1 << PB3);
	} else {
		refreshCounterValue = 1;
		PORTB |= (1 << PB3);
	}
}

//================================================================ functions ===
void fnMotorInit(void) {
	//CLK signal
	DDRB = (1 << PB3) | (1 << PB4);
	PORTB &= ~(1 << PB3);

	//CWCCW signal
	PORTB &= ~(1 << PB4);

	//CNTRL signal
	DDRD |= (1 << PD4) | (1 << PD5) | (1 << PD6);
	PORTD &= ~(1 << PD4);

	//EN signal
	PORTD |= (1 << PD5);

	//HF signal
	PORTD &= ~(1 << PD6);

	//initialize timer
	TCCR1B = (1 << WGM12) | (1 << CS11) | (1 << CS10); //CTC-Mode; Prescaler 64
}

void fnMoveToPosition(uint16_t endpoint) {
	busy = 1;

	currentStep = 0;
	refreshCounterValue = 0;
	goal = endpoint;

	//set direction
	if (goal > 0)
		PORTB |= (1 << PB4);
	else
		PORTB &= ~(1 << PB4);

	accelerationStop = (W_MAX * W_MAX) / (2 * ALPHA * WP_MAX);

	if (accelerationStop >= (goal / 2))
		accelerationStop = (goal / 2) - 2;

	decelerationStart = goal - accelerationStop - 2;

	counterValue = (F_CNTR * sqrtf((2 * ALPHA) / WP_MAX));
	counterValue = counterValue / 2;

	//set counter
	OCR1AH = (uint8_t) (counterValue >> 8);
	OCR1AL = (uint8_t) (counterValue & 0xFF);

	//multiply first counter value with correction value
	counterValue = counterValue * 0.676;

	//reset timer
	TCNT1H = 0x00;
	TCNT1L = 0x00;

	TIMSK |= (1 << OCIE1A);

	while(busy){
	}
}
